import { marked } from "marked";
function hasTokens(token) {
    return !!token.tokens || !!token.items;
}
const rules = {
    heading: /^(#{1,6})(\s+)(.+)(?:\n+|$)/,
    listItem: /^ *(?:[*+-]|\d+[.)])( .*)/,
    listItemStart: /^ *(?:[*+-]|\d+[.)]) /,
};
const tokenizer = {
    code() {
        //
    },
    list(src) {
        var _a;
        const match = (_a = src.split("\n").shift()) === null || _a === void 0 ? void 0 : _a.match(rules.listItem);
        if (match) {
            return {
                type: "list_item",
                raw: match[0],
                text: match[1].trimStart(),
            };
        }
        return false;
    },
};
function addPositions(token) {
    if (hasTokens(token)) {
        const subs = token.tokens || token.items;
        if (subs) {
            const start = token._start || 0;
            let subpos = 0;
            subs.forEach((sub) => {
                const substart = token.raw.indexOf(sub.raw, subpos);
                const sublen = sub.raw.length;
                sub._start = substart + start;
                sub._end = sub._start + sublen;
                subpos = substart + sublen;
            });
        }
    }
}
function getTokens(src) {
    marked.use({ tokenizer });
    const lexer = new marked.Lexer();
    const tokens = lexer.lex(src);
    // add position to each token
    const root = [
        {
            raw: src,
            tokens,
        },
    ];
    marked.walkTokens(root, addPositions);
    return tokens;
}
function walkTokens(tokens, fn) {
    marked.walkTokens(tokens, fn);
}
export { getTokens, walkTokens, rules };
